#!/usr/bin/env python

from setuptools import setup, find_packages
import os
import re
import subprocess

version_py = os.path.join(os.path.dirname(__file__), 'src/virgotools/', '_version.py')

def virgo_version():
    """
    This function atempts to get the version from the git tag or from the parent directory
    name and writes the version to a _version.py file for use later. It assumes the format
    of the tag is  v#r#p#.
    """

    #Try to get the version from git or from the directory

    try:
        version = subprocess.check_output(["git", "describe","--tag"]).decode('utf8').rstrip()
    except:
        version = os.path.basename(os.path.dirname(os.path.realpath(__file__)))

    #convert v1r2p3 to 1.2.3 and v4r5 to 4.5.0
    match = re.match('^v(\d+)r(\d+)(?:p(\d+))?$', version)
    if match:
        version = '.'.join(match.groups('0'))  # assume p0 if missing
    elif(os.getenv("PKG_VERSION") is not None): # If in a conda environment
        version = os.getenv("PKG_VERSION")
    elif(os.path.exists(version_py)):
        with open(version_py, 'r') as f:
            version = f.read().strip().split('=')[-1].replace('"','')
    else:
        return "99.99.99"

    if(match or os.getenv("PKG_VERSION") is not None):
        #Write the version to a file so can be used later
        version_msg = "# Do not edit this file, pipeline versioning is governed by git tags"
        with open(version_py, 'w') as f:
            f.write(version_msg + os.linesep + ("__version__=\"%s\"" % version))

    return version



setup(
    name='PythonVirgoTools',
    version=virgo_version(),
    description='tools for interfacing with the Virgo interferometer',
    author='Bas Swinkels',
    author_email='swinkels@nikhef.nl',
    url='http://www.virgo-gw.eu/',
    package_dir={'': 'src'},
    install_requires=[
        'numpy',
        'pytz',
        'future',
        'tzlocal',
#        'pytango',  #keeps rebuilding even if wheel exists, better to build by hand
#        'PyCm',  # different versions between Py2 and Py3, so install by hand
#        'PyFd',  # idem
    ],
    # tests_require does not work with pip??
    extras_require={
        'test': [
            'pytest',
            'mock',
        ],
    },
    package_data = {'virgotools':['leap-seconds.list']},
    packages=find_packages('src'),
)
