from builtins import range
from builtins import object
import logging
import time
import ctypes

import PyFd as fd
import PyCm as cm
from virgotools.frame_lib import Frame
from virgotools import cm_lib

__all__ = ['ConnectOnline']

logger = logging.getLogger(__name__)


# hard-coded parameters for connecting online, ask Didier
COMPRESS = -1
N_RETRY = 2
QUEUESIZE = 2


# FIXME: test, interface might still change
class ConnectOnline(object):
    """Connect online to server to get live frames, similar to dataDisplay
    """
    def __init__(self, server, tag):
        self.server = server
        self.current_frame = None
        self._last_gps = None  # for calculating Frame.gap

        if cm_lib._my_cm_name is None:
            raise cm_lib.CmError("I do not have a cm-name, use cm_start first")

        @cm_lib.cm_handler
        def handle_FdFrame(message, sender, server):
            if self.current_frame:
                self.current_frame.close()

            nBytes = ctypes.c_int()
            p_frame = fd.FdGetFrameFromMessage(message, COMPRESS,
                                               ctypes.byref(nBytes))
            assert p_frame, 'got no frame from message'

            self.current_frame = Frame(p_frame)
            logger.debug('Got frame with gps = %i', self.current_frame.gps)
            return cm.CmMessageBreak

        # prevent handler from being garbage collected, which causes segfault
        self._handler = handle_FdFrame

        logger.info('Installing handler for FdFrame')
        cm.CmMessageInstallHandler(handle_FdFrame, 'FdFrame')

        logger.info('Asking %s to send frames', server)
        fd.FdRequestFrames(server, cm_lib._my_cm_name, tag, QUEUESIZE, N_RETRY)

    def get_next_frame(self, timeout=10):
        if self.current_frame:
            old_gps = self.current_frame.gps
        else:
            old_gps = -1
        t = time.time()
        while time.time() - t < timeout:
            cm.CmMessageWaitWithTimeout(0.1)
            if self.current_frame and self.current_frame.gps != old_gps:
                self._last_gps = self.current_frame._calculate_gap(
                    self._last_gps)
                return self.current_frame

        msg = 'Did not get any frame after %i seconds' % timeout
        raise ValueError(msg)  # FIXME: FrameError?

    def wait(self, dt):
        logger.info('Waiting for %g seconds', dt)
        for _ in range(int(dt)):  # assume frames are 1 second for now
            self.get_next_frame()

    def disconnect(self):
        logger.info('Asking %s to stop sending frames', self.server)
        fd.FdRemoveFramesRequest(self.server, cm_lib._my_cm_name)

        # wait for an answer
        cm.CmMessageWaitWithTimeout(1.0)

        logger.info('Uninstalling handler for FdFrame')
        cm.CmMessageUninstallHandler('FdFrame')

    # context manager protocol
    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.disconnect()
